#include <errno.h>
#include <string>
#include <sstream>
#include <string.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "ApproachSearchDialog.h"
#include "ApproachPlate.h"
#include "Airport.h"
#include "Util.h"

ApproachSearchDialog::ApproachSearchDialog(std::string *GladeFilename, DataStore *Data, Config *Cfg)
{
	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "ApproachSearchDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error reading data file.  Program cannot continue.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("ApproachSearchDialog",GtkDialog);
	refXml->get_widget("AirportEntry",AirportEntry);
	refXml->get_widget("ApproachesTreeView",ApproachesTreeView);
	refXml->get_widget("ViewButton",ViewButton);
	refXml->get_widget("FrontMatterButton",FrontMatterButton);
	refXml->get_widget("CloseButton",CloseButton);
	refXml->get_widget("OnScreenKeypadParent",OnScreenKeypadParent);

	Keypad=new OnScreenKeypad();
	OnScreenKeypadParent->add(*Keypad);
	OnScreenKeypadParent->reorder_child(*Keypad,1);
	Keypad->show();
	Keypad->set_target(AirportEntry);

	AirportEntry->signal_changed().connect(sigc::mem_fun(this,&ApproachSearchDialog::on_AirportEntry_changed));
	ViewButton->signal_clicked().connect(sigc::mem_fun(this,&ApproachSearchDialog::on_ViewButton_clicked));
	FrontMatterButton->signal_clicked().connect(sigc::mem_fun(this,&ApproachSearchDialog::on_FrontMatterButton_clicked));
	CloseButton->signal_clicked().connect(sigc::mem_fun(this,&ApproachSearchDialog::on_CloseButton_clicked));

	this->Cfg=Cfg;
	this->Data=Data;

	MatchingItems.clear();

	ListStore = Gtk::ListStore::create(Columns);
	ApproachesTreeView->set_model(ListStore);
	ApproachesTreeView->append_column("Plate", Columns.PlateName);
	ApproachesTreeView->append_column("ID", Columns.AirportIdent);
	ApproachesTreeView->append_column("Name", Columns.AirportName);

	Gtk::TreeView::Column* pColumn = ApproachesTreeView->get_column(0);
	if(pColumn)
		pColumn->set_sort_column(Columns.AirportIdent);

	pColumn = ApproachesTreeView->get_column(1);
	if(pColumn)
		pColumn->set_sort_column(Columns.AirportName);

	pColumn = ApproachesTreeView->get_column(2);
	if(pColumn)
		pColumn->set_sort_column(Columns.PlateName);
}

ApproachSearchDialog::~ApproachSearchDialog()
{
	delete GtkDialog;
}

int ApproachSearchDialog::Run() {
	if (!Data->HasActiveVolume()) {
		Gtk::MessageDialog *Message = new Gtk::MessageDialog("No active data volume",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		Message->run();
		delete Message;
		return 0;
	}

	return GtkDialog->run();
}

void ApproachSearchDialog::on_AirportEntry_changed() {
	static std::string OldText,CurrText;
	std::vector<Airport*>::iterator Airporti;
	std::vector<ApproachPlate*>::iterator Approachi;
	std::vector<MatchingItem>::iterator MatchingItemi;

	if (!Data->HasActiveVolume())
		return;	/* No data to search */

	CurrText=AirportEntry->get_text();

	if (CurrText.length()<=1 || (CurrText.length()<=2 && (toupper(CurrText[0])=='K'))) {
		MatchingItems.clear();
		OldText=CurrText;
	} else {
		if (CurrText.substr(0,CurrText.length()-1)==OldText && (MatchingItems.size()>0 && CurrText.length()>2)) {
			/* The new search is a subset of the old one */
			std::vector<MatchingItem> NewMatchingItems;
			for (MatchingItemi=MatchingItems.begin();MatchingItemi!=MatchingItems.end();MatchingItemi++) {
				if ((*MatchingItemi).Matchi(CurrText)) {
					NewMatchingItems.push_back(*MatchingItemi);
				}
			}
			MatchingItems=NewMatchingItems;
		} else {
			/* We have to search all of the approaches */
			MatchingItems.clear();
			for (Airporti=Data->GetAirports()->begin();Airporti!=Data->GetAirports()->end();++Airporti) {
				if ((*Airporti)->Matchi(CurrText)) {
					/* The airport matches - get all of its approaches */
					for (Approachi=(*Airporti)->GetApproachPlates()->begin();Approachi!=(*Airporti)->GetApproachPlates()->end();++Approachi) {
						MatchingItem NewItem((*Airporti),(*Approachi));
						MatchingItems.push_back(NewItem);
						if (MatchingItems.size()>64) {
							/* We have too many matches - give up to avoid bogging down slower computers */
							MatchingItems.clear();
							ListStore->clear();
							OldText=CurrText;
							return;
						}
					}
				}
			}
		}
	}

	ListStore->clear();

	for (MatchingItemi=MatchingItems.begin();MatchingItemi!=MatchingItems.end();MatchingItemi++) {
		Gtk::TreeModel::iterator iter = ListStore->append();
		Gtk::TreeModel::Row row = *iter;

		row[Columns.PlateName] = (*MatchingItemi).GetPlateName();
		row[Columns.AirportIdent] = (*MatchingItemi).GetAirportIdent();
		row[Columns.AirportName] = (*MatchingItemi).GetAirportName();
		row[Columns.ImgName] = (*MatchingItemi).GetImgName();
	}

	if (MatchingItems.size()) {
		/* Select the first item by default */
		Glib::RefPtr<Gtk::TreeSelection> ApproachSelection = ApproachesTreeView->get_selection();
		Gtk::TreeModel::iterator iter = ListStore->children().begin();
		if(iter)
			ApproachSelection->select(iter);
	}

	OldText=CurrText;
}

void ApproachSearchDialog::on_ViewButton_clicked() {
	Glib::RefPtr<Gtk::TreeSelection> ApproachSelection = ApproachesTreeView->get_selection();
	Gtk::TreeModel::iterator iter = ApproachSelection->get_selected();
	if (iter) {		/* If anything is selected */
		Gtk::TreeModel::Row row = *iter;
		std::string Filename;
		pid_t forkresult;

		Filename=(*(Data->GetActiveVolume()))->GetDirectory()+"/plateimages/";
		Filename+=row[Columns.ImgName];
		Util::ShellExpand(&Filename);

		forkresult=fork();

		if (forkresult==-1) {
			std::ostringstream ErrMsg;
			ErrMsg<<"Unable to fork(): "<<strerror(errno)<< " ("<<errno<<")";
			Gtk::MessageDialog *message = new Gtk::MessageDialog(ErrMsg.str(),
				false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
			message->run();
			delete message;

			return;
		}
		if (forkresult==0) {
			execlp("evince","evince",Filename.c_str(),(char *) NULL);
			fprintf(stderr,"Error executing '%s \"%s\"': %s (%u)\n","evince",Filename.c_str(),strerror(errno),errno);
			Gtk::Main::quit();
		}
	}
}

void ApproachSearchDialog::on_FrontMatterButton_clicked() {
	std::string Filename;
	pid_t forkresult;

	Filename=(*(Data->GetActiveVolume()))->GetDirectory()+"/plateimages/"+FRONTMATTERFILENAME;
	Util::ShellExpand(&Filename);

	forkresult=fork();

	if (forkresult==-1) {
		std::ostringstream ErrMsg;
		ErrMsg<<"Unable to fork(): "<<strerror(errno)<< " ("<<errno<<")";
		Gtk::MessageDialog *message = new Gtk::MessageDialog(ErrMsg.str(),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		delete message;

		return;
	}
	if (forkresult==0) {
		execlp("evince","evince",Filename.c_str(),(char *) NULL);
		fprintf(stderr,"Error executing '%s \"%s\"': %s (%u)\n","evince",Filename.c_str(),strerror(errno),errno);
		Gtk::Main::quit();
	}
}

void ApproachSearchDialog::on_CloseButton_clicked() {
	GtkDialog->response(Gtk::RESPONSE_NONE);
}

ApproachSearchDialogModelColumns::ApproachSearchDialogModelColumns() {
	add(PlateName);
	add(AirportIdent);
	add(AirportName);
	add(ImgName);
}

ApproachSearchDialog::MatchingItem::MatchingItem(Airport *AirP, ApproachPlate *Plate) {
	this->AirP=AirP;
	this->Plate=Plate;
}

bool ApproachSearchDialog::MatchingItem::Matchi(std::string str) {
	return AirP->Matchi(str);
}

std::string ApproachSearchDialog::MatchingItem::GetAirportIdent() {
	return AirP->GetICAO();
}

std::string ApproachSearchDialog::MatchingItem::GetAirportName() {
	return AirP->GetName();
}

std::string ApproachSearchDialog::MatchingItem::GetPlateName() {
	return Plate->GetName();
}

std::string ApproachSearchDialog::MatchingItem::GetImgName() {
	return Plate->GetImgName();
}
