#include "CRC32.h"

/* This code was adapted from http://www.createwindow.com/programming/crc32/index.htm
 * It is believed to be free (Public Domain)
 */

CRC32::CRC32() {
	/* Initialize the CRC table */

	/* This is the official polynomial used by CRC-32
	 * in PKZip, WinZip and Ethernet.
	 */
	uint32_t Polynomial = 0x04c11db7;

	/* 256 values representing ASCII character codes. */
	for(int i = 0; i <= 0xFF; i++) {
		LookupTable[i]=Reflect(i, 8) << 24;
		for (int j = 0; j < 8; j++)
			LookupTable[i] = (LookupTable[i] << 1) ^ (LookupTable[i] & (1 << 31) ? Polynomial : 0);
		LookupTable[i] = Reflect(LookupTable[i], 32);
	}
}

/* Used only by the constructor. */
uint32_t CRC32::Reflect(uint32_t ref, char ch) {
	uint32_t value=0;

	/* Swap bit 0 for bit 7
	 * bit 1 for bit 6, etc.
	 */
	for(int i = 1; i < (ch + 1); i++) {
		if (ref & 1)
			value |= 1 << (ch - i);
		ref >>= 1;
	}
	return value;
}

/* Pass a data buffer and size to this function and it will return the CRC.
 * 
 * Once the lookup table has been filled in,
 * this function creates all CRCs using only the lookup table.
 *
 * Be sure to use unsigned variables,
 * because negative values introduce high bits
 * where zero bits are required.
 */
uint32_t CRC32::Compute(void *data, int len, uint32_t RunningCRC) {
	unsigned char* buffer=(unsigned char*)data;

	/* Perform the algorithm on each character
	 * in the string, using the lookup table values.
	 */
	while (len--) {
		RunningCRC = (RunningCRC >> 8) ^ LookupTable[(RunningCRC & 0xFF)^ *buffer++];
	}

	return RunningCRC;
}

uint32_t CRC32::Finalize(uint32_t CRC) {
	/* Exclusive OR the result with the beginning value. */
	return CRC ^ 0xffffffff;
}
