#define __STDC_LIMIT_MACROS
#include <sstream>
#include <math.h>
#include <string.h>

#include "Frequency.h"
#include "ARINC424_18.h"

const std::string Frequency::UnitsText[4]= {
	"Hz",
	"KHz",
	"MHz",
	"GHz"
};

Frequency::Frequency(unsigned int RawFreq, unsigned int Precision, UnitsEnum Units, TypeEnum Type) {
	this->RawFreq=RawFreq;
	this->Precision=Precision;
	this->Units=Units;
	this->Type=Type;
}

double Frequency::ToDouble() {
	return RawFreq/pow(Precision,10.0);
}

std::string Frequency::FreqToString() {
	unsigned int p, factor, whole, decimal;
	std::ostringstream result;

	/* 12345,2 -> 123.45 */

	factor=1;
	for (p=0;p<Precision;p++) {
		factor*=10;
	}

	whole=RawFreq/factor;
	decimal=RawFreq%factor;

	result<<whole;
	if (Precision>0) {
		result<<"."<<decimal;
	}

	return result.str();
}

std::string Frequency::ToString() {
	return FreqToString()+" "+Frequency::UnitsText[Units];
}

/* Accessors */
unsigned int Frequency::GetRawFreq() {
	return RawFreq;
}

void Frequency::SetRawFreq(unsigned int RawFreq) {
	this->RawFreq=RawFreq;
}

unsigned char Frequency::GetPrecision() {
	return Precision;
}

void Frequency::SetPrecision(unsigned char Precision) {
	this->Precision=Precision;
}

enum Frequency::UnitsEnum Frequency::GetUnits() {
	return Units;
}

void Frequency::SetUnits(UnitsEnum Units) {
	this->Units=Units;
}

enum Frequency::TypeEnum Frequency::GetType() {
	return Type;
}

void Frequency::SetType(TypeEnum Type) {
	this->Type=Type;
}

std::string Frequency::GetDesc() {
	return Desc;
}

void Frequency::SetDesc(std::string Desc) {
	this->Desc=Desc;
}

std::string Frequency::GetAirportICAO()
{
	return AirportICAO;
}

bool Frequency::ParseNFD(std::string Line)
{
	struct arinc424_18_comm_struct *NFDData;
	char strbuf[sizeof(struct arinc424_18_comm_struct)];

	if (Line.length()!=sizeof(struct arinc424_18_comm_struct)+1)	/* Allow for LF */
	{
		return false;
	}

	if (strcmp(arinc424_18_parse_recordtype(Line.c_str()),ARINC424_18_RECTYPE_AIRPORT_COMMUNICATIONS)!=0)
	{
		return false;
	}

	NFDData=(struct arinc424_18_comm_struct *)Line.c_str();

	/* ICAO */
	memcpy(strbuf,NFDData->header.airportid,sizeof(NFDData->header.airportid));
	strbuf[sizeof(NFDData->header.airportid)]=0;
	AirportICAO=strbuf;

	/* Frequency */
	RawFreq=arinc424_18_parse_freq(NFDData->header.freq);
	Precision=3;
	Units=MHZ;
	Type=MISC;

	return true;
}

void Frequency::Clone(Frequency *SrcFreq)
{
	RawFreq=SrcFreq->GetRawFreq();
	Precision=SrcFreq->GetPrecision();
	Units=SrcFreq->GetUnits();
	Type=SrcFreq->GetType();
	Desc=SrcFreq->GetDesc();
	AirportICAO=SrcFreq->GetAirportICAO();
}

bool Frequency::MakeBinary(struct Frequency_Binary_struct *BinaryData) {
	/* Sanity checking */
	if (
		RawFreq>UINT32_MAX || RawFreq<0 ||
		Units>UINT8_MAX || Units<0 ||
		Type>UINT8_MAX || Type<0 ||
		Desc.size()>sizeof(BinaryData->Desc)
		) {
		return false;
	}

	BinaryData->RawFreq=RawFreq;
	BinaryData->Precision=Precision;
	BinaryData->Units=Units;
	BinaryData->Type=Units;
	strncpy(BinaryData->Desc,Desc.c_str(),sizeof(BinaryData->Desc));

	return true;
}

void Frequency::SetFromBinary(struct Frequency_Binary_struct *BinaryData) {
	RawFreq=BinaryData->RawFreq;
	Precision=BinaryData->Precision;
	Units=(Frequency::UnitsEnum)BinaryData->Units;
	Type=(Frequency::TypeEnum)BinaryData->Type;
	Desc.assign(BinaryData->Desc,sizeof(BinaryData->Desc));
	/* Reset the size to the actual number of characters */
	Desc=Desc.c_str();
}

