#include "ImportSectionalsDVDVolumeChooserDialog.h"

#ifndef _GNU_SOURCE
/* Needed for strstr() in string.h */
#define _GNU_SOURCE
#endif

#ifndef _XOPEN_SOURCE
/* Needed for strptime() in time.h */
#define _XOPEN_SOURCE
#endif

#include <string.h>
#include <glob.h>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "Exception.h"
#include "Util.h"
#include "DataStore.h"

ImportSectionalsDVDVolumeChooserDialog::ImportSectionalsDVDVolumeChooserDialog(std::string *GladeFilename, std::string DataPath)
{
	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "ImportSectionalsDVDVolumeChooserDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error reading data file.  Program cannot continue.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("ImportSectionalsDVDVolumeChooserDialog",GtkDialog);
	refXml->get_widget("VolumeComboBox",VolumeComboBox);
	refXml->get_widget("EffectiveFromLabel",EffectiveFromLabel);
	refXml->get_widget("EffectiveToLabel",EffectiveToLabel);

	VolumeComboBox->signal_changed().connect(sigc::mem_fun(this,&ImportSectionalsDVDVolumeChooserDialog::on_VolumeComboBox_changed));

	/* Setup the Volume ComboBox */
	VolumeComborefTreeModel = Gtk::ListStore::create(VolumeComboColumns);
	VolumeComboBox->set_model(VolumeComborefTreeModel);

	this->DataPath=DataPath;
}

ImportSectionalsDVDVolumeChooserDialog::~ImportSectionalsDVDVolumeChooserDialog() {
	delete GtkDialog;
}

int ImportSectionalsDVDVolumeChooserDialog::Run() {
	PopulateVolumeComboBox();

	return GtkDialog->run();
}

void ImportSectionalsDVDVolumeChooserDialog::GetData(std::string *Volume, time_t *EffectiveFrom, time_t *EffectiveTo) {
	*Volume=this->Volume;
	*EffectiveFrom=this->EffectiveFrom;
	*EffectiveTo=this->EffectiveFrom+EffectiveDuration-1;
}

void ImportSectionalsDVDVolumeChooserDialog::on_VolumeComboBox_changed() {
	Gtk::TreeModel::iterator Activei;
	char buf[256];
	struct tm bdtime;
	time_t EffectiveTo;

	Activei=VolumeComboBox->get_active();

	if (Activei) {		/* If anything is selected */
		Gtk::TreeModel::Row row = *Activei;
		Volume=row[VolumeComboColumns.Volume];
		EffectiveFrom=row[VolumeComboColumns.EffectiveFrom];

		gmtime_r(&EffectiveFrom,&bdtime);
		snprintf(buf,sizeof(buf),"%02u/%02u/%02u",bdtime.tm_mon+1,bdtime.tm_mday,bdtime.tm_year-100);
		EffectiveFromLabel->set_text(buf);

		EffectiveTo=EffectiveFrom+EffectiveDuration;
		gmtime_r(&EffectiveTo,&bdtime);
		snprintf(buf,sizeof(buf),"%02u/%02u/%02u",bdtime.tm_mon+1,bdtime.tm_mday,bdtime.tm_year-100);
		EffectiveToLabel->set_text(buf);
	} else {
		/* Nothing selected */
		EffectiveFrom=0;
		EffectiveFromLabel->set_text("");
		EffectiveToLabel->set_text("");
	}
}

void ImportSectionalsDVDVolumeChooserDialog::PopulateVolumeComboBox() {
	glob_t globdata;
	std::ifstream ifs;
	char buf[4096];
	std::string MinLine,MaxLine;
	time_t MetaFrom,MetaTo;
	struct tm bdfrom,bdto;
	int VolYear, VolIndex;


	/* Pick the first .html file to parse for data */
	if (glob((DataPath+"/*.htm*").c_str(),GLOB_NOSORT,NULL,&globdata)!=0) {
		throw Exception("Error reading metadata",Exception::LEVEL_ERROR);
	}
	
	if (globdata.gl_pathc==0) {
		globfree(&globdata);
		throw Exception("Missing metadata",Exception::LEVEL_ERROR);
	}

	ifs.open(globdata.gl_pathv[0],std::ifstream::in);
	globfree(&globdata);
	if (!ifs.good()) {
		throw Exception("Error opening metadata",Exception::LEVEL_ERROR);
	}

	do {
		ifs.getline(buf,sizeof(buf));
		if (strstr(buf,"dc.coverage.t.min")) {
			MinLine=buf;
		} else if (strstr(buf,"dc.coverage.t.max")) {
			MaxLine=buf;
		}
	} while (ifs.good() && (MinLine.length()==0 || MaxLine.length()==0));

	if (!ParseMetaDate(MinLine,&MetaFrom)) {
		throw Exception("Error parsing metadata",Exception::LEVEL_ERROR);
	}

	if (!ParseMetaDate(MaxLine,&MetaTo)) {
		throw Exception("Error parsing metadata",Exception::LEVEL_ERROR);
	}
	/* The "To" data needs to be adjusted forward 1 day minus 1 second to be consisted
	 * with the way the rest of the Effective dates are handled
	 */
	MetaTo=MetaTo+24*60*60-1;

	/* Scan through and set the possible volumes */
	VolumeComboBox->clear();

	gmtime_r(&MetaFrom,&bdfrom);
	gmtime_r(&MetaTo,&bdto);
	for (VolYear=bdfrom.tm_year-100;VolYear<=bdto.tm_year-100;VolYear++) {
		for (VolIndex=1;VolIndex<=14;VolIndex++) {
			std::ostringstream OVolStr;
			OVolStr<<std::setfill('0')<<std::setw(2)<<VolYear<<std::setw(2)<<VolIndex;
			if (
				MetaFrom<=Volume::ComputeEffectiveFrom(OVolStr.str()) &&
				MetaTo>=Volume::ComputeEffectiveTo(OVolStr.str())
			) {
				Gtk::TreeModel::Row row = *(VolumeComborefTreeModel->append());
				row[VolumeComboColumns.Volume] = OVolStr.str();
				row[VolumeComboColumns.EffectiveFrom] = Volume::ComputeEffectiveFrom(OVolStr.str());
			}
		}
	}

	VolumeComboBox->pack_start(VolumeComboColumns.Volume);
	VolumeComboBox->set_active(0);
}

bool ImportSectionalsDVDVolumeChooserDialog::ParseMetaDate(std::string Line, time_t *Date) {
	struct tm bdtime;

	if (Line.length()==0)
		return false;

	Line=Line.substr(Line.find("content="));		/* Strip off everything before 'content=' */
	Line=Line.substr(Line.find_first_of('\"')+1);	/* Strip off the leading 'content="' */
	Line=Line.erase(Line.find_first_of('\"'));		/* Strip off everything after the closing quote */
	/* Line is now in the form of 'FullMonthName, MDay, Year' */

	memset(&bdtime,0,sizeof(bdtime));
	strptime(Line.c_str(),"%B %d, %Y",&bdtime);

	/* Things become effective at 0901Z */
	bdtime.tm_hour=9;
	bdtime.tm_min=1;

	*Date=Util::GMMkTime(&bdtime);
	return true;
}
