#ifndef NAVAID_H_
#define NAVAID_H_

#include <stdint.h>
#include "Waypoint.h"
#include "Frequency.h"

class NavAid : public Waypoint
{
public:
	enum Type_enum {
		TYPE_VOR,
		TYPE_NDB,
		TYPE_ILS,
		TYPE_LOCALIZER
	};

	NavAid();
	virtual ~NavAid();

	virtual enum Type_enum GetType()=0;
	Frequency *GetFreq();
	void SetFreq(Frequency *NewFreq);
	double GetDeclination();	/* 
								 * Appears to be the magnetic variation it was calibrated to
								 * Positive=East, Negative=West
								 * */
	virtual bool ParseNFD(std::string Line);
	bool MakeBinary(struct NavAid_Binary_struct *BinaryData);
	void SetFromBinary(struct NavAid_Binary_struct *BinaryData);

protected:
	Frequency Freq;
	double Declination;
};

class VOR: public NavAid
{
public:
	VOR();
	enum NavAid::Type_enum GetType();
	bool ParseNFD(std::string Line);
};

class NDB: public NavAid
{
public:
	NDB();
	enum NavAid::Type_enum GetType();
	bool ParseNFD(std::string Line);
};

class ILS: public NavAid
{
public:
	ILS();
	enum NavAid::Type_enum GetType();
};

class Localizer: public NavAid
{
public:
	Localizer();
	enum NavAid::Type_enum GetType();
};

/* Increment the version when the structure changes */
#define NAVAID_BINARY_STRUCT_VER 1
struct NavAid_Binary_struct {
	char ID[16];
	char Name[50];
	int32_t Lonx1000000,Latx1000000,Declinationx100;
	uint8_t Type;
} __attribute__ ((packed));

#endif /*NAVAID_H_*/
