#ifndef ROUTEDIALOG_H_
#define ROUTEDIALOG_H_

#include <gtkmm/dialog.h>
#include <gtkmm/button.h>
#include <gtkmm/box.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/scrolledwindow.h>
#include <libglademm.h>
#include <string>
#include "DataStore.h"
#include "Route.h"
#include "RoutePoint.h"
#include "Aircraft.h"

class RouteDialogModelColumns : public Gtk::TreeModelColumnRecord
{
public:
	RouteDialogModelColumns();

	Gtk::TreeModelColumn<std::string> ID;
	Gtk::TreeModelColumn<std::string> LegDist;
	Gtk::TreeModelColumn<std::string> CumDist;
	Gtk::TreeModelColumn<std::string> LegTime;
	Gtk::TreeModelColumn<std::string> CumTime;
	Gtk::TreeModelColumn<std::list<RoutePoint*>::iterator> RoutePointi;
};

class RouteDialog
{
public:
	RouteDialog(std::string *GladeFilename, DataStore *Data, Route *RouteData, Aircraft *AircraftData);
	~RouteDialog();
	int Run();

private:
	std::string *GladeFilename;
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Dialog *GtkDialog;
	Gtk::Button *CloseButton;
	Gtk::Button *NewButton;
	Gtk::Button *ReverseButton;
	Gtk::Button *UpButton;
	Gtk::Button *DownButton;
	Gtk::Button *DeleteButton;
	Gtk::ScrolledWindow *ScrolledWindow;
	Gtk::TreeView *RouteTreeView;

	RouteDialogModelColumns Columns;
	Glib::RefPtr<Gtk::ListStore> ListStore;
	DataStore *Data;
	Route *RouteData;
	Aircraft *AircraftData;

	void on_CloseButton_clicked();
	void on_NewButton_clicked();
	void on_ReverseButton_clicked();
	void on_UpButton_clicked();
	void on_DownButton_clicked();
	void on_DeleteButton_clicked();
	void on_TreeView_cursor_changed();

	void PopulateTreeView(std::list<RoutePoint*>::iterator Selectedi);
	void AddRoutePoint(Gtk::TreeModel::Row *Row, std::list<RoutePoint*>::iterator RPi, double LegDist, double CumDist, bool First=false, bool Last=false);
};

#endif /*ROUTEDIALOG_H_*/
