#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "ARINC424_18.h"
#include "Runway.h"

Runway::Runway()
{
}

/* Accessors */
unsigned int Runway::GetLength() {
	return Length;
}

void Runway::SetLength(unsigned int Length) {
	this->Length=Length;
}

unsigned int Runway::GetWidth() {
	return Width;
}

void Runway::SetWidth(unsigned int Width) {
	this->Width=Width;
}

unsigned int Runway::GetDisplaced() {
	return Displaced;
}

void Runway::SetDisplaced(unsigned int Displaced) {
	this->Displaced=Displaced;
}

unsigned int Runway::GetElevation() {
	return Elevation;
}

void Runway::SetElevation(unsigned int Elevation) {
	this->Elevation=Elevation;
}

double Runway::GetMagBearing() {
	return MagBearing;
}

void Runway::SetMagBearing(double MagBearing) {
	this->MagBearing=MagBearing;
}

double Runway::GetLon() {
	return Lon;
}

void Runway::SetLon(double Lon) {
	this->Lon=Lon;
}

double Runway::GetLat() {
	return Lat;
}

void Runway::SetLat(double Lat) {
	this->Lat=Lat;
}

double Runway::GetGradient() {
	return Gradient;
}

void Runway::SetGradient(double Gradient) {
	this->Gradient=Gradient;
}

std::string Runway::GetNotes() {
	return Notes;
}

void Runway::SetNotes(std::string Notes) {
	this->Notes=Notes;
}

std::string Runway::GetID() {
	return ID;
}

void Runway::SetID(std::string ID) {
	this->ID=ID;
}

void Runway::ClearNavAids() {
	NavAids.clear();
}

void Runway::AddNavAid(NavAid *newNavAid) {
	NavAids.push_back(newNavAid);
}

std::vector<NavAid*> Runway::GetNavAids() {
	return NavAids;
}

std::string Runway::GetAirportICAO()
{
	return AirportICAO;
}

bool Runway::ParseNFD(std::string Line)
{
	struct arinc424_18_runway_struct *NFDData;
	char strbuf[sizeof(struct arinc424_18_runway_struct)];

	if (Line.length()!=sizeof(struct arinc424_18_runway_struct)+1)	/* Allow for LF */
	{
		return false;
	}

	if (strcmp(arinc424_18_parse_recordtype(Line.c_str()),ARINC424_18_RECTYPE_AIRPORT_RUNWAY)!=0)
	{
		return false;
	}
	NFDData=(struct arinc424_18_runway_struct *)Line.c_str();

	/* Airport ICAO */
	memcpy(strbuf,NFDData->header.airportid,sizeof(NFDData->header.airportid));
	strbuf[sizeof(NFDData->header.airportid)]=0;
	AirportICAO=strbuf;

	/* Runway ID */
	memcpy(strbuf,NFDData->header.id,sizeof(NFDData->header.id));
	strbuf[sizeof(NFDData->header.id)]=0;
	ID=strbuf;

	/* Latitude */
	Lat=arinc424_18_parse_lat(NFDData->lat);
	if (isnan(Lat))
		return false;

	/* Longitude */
	Lon=arinc424_18_parse_lon(NFDData->lon);
	if (isnan(Lon))
		return false;

	/* Length */
	memcpy(strbuf,NFDData->length,sizeof(NFDData->length));
	strbuf[sizeof(NFDData->length)]=0;
	Length=atoi(strbuf);

	/* Width */
	memcpy(strbuf,NFDData->width,sizeof(NFDData->width));
	strbuf[sizeof(NFDData->width)]=0;
	Width=atoi(strbuf);

	/* Elevation */
	memcpy(strbuf,NFDData->thresholdelevation,sizeof(NFDData->thresholdelevation));
	strbuf[sizeof(NFDData->thresholdelevation)]=0;
	Elevation=atoi(strbuf);

	/* Magnetic Bearing */
	memcpy(strbuf,NFDData->magbearing,sizeof(NFDData->magbearing));
	strbuf[sizeof(NFDData->magbearing)]=0;
	MagBearing=atoi(strbuf)/10.0;

	/* Gradient */
	memcpy(strbuf,NFDData->gradient,sizeof(NFDData->gradient));
	strbuf[sizeof(NFDData->gradient)]=0;
	if (strbuf[0]==' ')
		/* Not provided */
		Gradient=NAN;
	else
		Gradient=atoi(strbuf)/1000.0;

	/* Displaced Threshold */
	memcpy(strbuf,NFDData->displacedthresholddist,sizeof(NFDData->displacedthresholddist));
	strbuf[sizeof(NFDData->displacedthresholddist)]=0;
	Displaced=atoi(strbuf);

	return true;
}

bool Runway::MakeBinary(struct Runway_Binary_struct *BinaryData) {
	double MagBearingx10,Lonx1000000,Latx1000000,Gradientx10;

	MagBearingx10=MagBearing*10;
	Lonx1000000=Lon*1000000;
	Latx1000000=Lon*1000000;
	Gradientx10=Gradient*10;
	/* Sanity checking */
	if (
		Length>UINT16_MAX ||
		Width>UINT16_MAX ||
		Displaced>UINT16_MAX ||
		Elevation<INT16_MIN || Elevation>INT16_MAX ||
		MagBearingx10<INT16_MIN || MagBearingx10>INT16_MAX ||
		Lonx1000000<INT32_MIN || Lonx1000000>INT32_MAX ||
		Latx1000000<INT32_MIN || Latx1000000>INT32_MAX ||
		Gradientx10<INT16_MIN || Gradientx10>INT16_MAX ||
		Notes.size()>sizeof(BinaryData->Notes) ||
		ID.size()>sizeof(BinaryData->ID)
		) {
		return false;
	}

	BinaryData->Length=Length;
	BinaryData->Width=Width;
	BinaryData->Displaced=Displaced;
	BinaryData->Elevation=Elevation;
	BinaryData->MagBearingx10=(uint16_t)round(MagBearing*10);
	BinaryData->Lonx1000000=(int32_t)round(Lon*1000000);
	BinaryData->Latx1000000=(int32_t)round(Lat*1000000);
	BinaryData->Gradientx10=(int16_t)round(Gradient*10);
	strncpy(BinaryData->Notes,Notes.c_str(),sizeof(BinaryData->Notes));
	strncpy(BinaryData->ID,ID.c_str(),sizeof(BinaryData->ID));

	return true;
}

void Runway::SetFromBinary(struct Runway_Binary_struct *BinaryData) {
	Length=BinaryData->Length;
	Width=BinaryData->Width;
	Displaced=BinaryData->Displaced;
	Elevation=BinaryData->Elevation;
	MagBearing=BinaryData->MagBearingx10/10.0;
	Lon=BinaryData->Lonx1000000/1000000.0;
	Lat=BinaryData->Latx1000000/1000000.0;
	Gradient=BinaryData->Gradientx10/10.0;
	Notes.assign(BinaryData->Notes,sizeof(BinaryData->Notes));
	ID.assign(BinaryData->ID,sizeof(BinaryData->ID));
	/* Reset the size to the actual number of characters */
	Notes=Notes.c_str();
	ID=ID.c_str();
}

