#ifndef RUNWAY_H_
#define RUNWAY_H_

#include <string>
#include <vector>
#include "NavAid.h"

class Runway
{
public:
	Runway();

	/* Accessors */
	unsigned int GetLength();
	void SetLength(unsigned int Length);
	unsigned int GetWidth();
	void SetWidth(unsigned int Width);
	unsigned int GetDisplaced();
	void SetDisplaced(unsigned int Displaced);
	unsigned int GetElevation();
	void SetElevation(unsigned int Elevation);
	double GetMagBearing();
	void SetMagBearing(double MagBearing);
	double GetLon();
	void SetLon(double Lon);
	double GetLat();
	void SetLat(double Lat);
	double GetGradient();
	void SetGradient(double Gradient);
	std::string GetNotes();
	void SetNotes(std::string Notes);
	std::string GetID();
	void SetID(std::string ID);
	void ClearNavAids();
	void AddNavAid(NavAid *newNavAid);
	std::vector<NavAid*> GetNavAids();
	std::string GetAirportICAO();
	bool ParseNFD(std::string Line);
	bool MakeBinary(struct Runway_Binary_struct *BinaryData);
	void SetFromBinary(struct Runway_Binary_struct *BinaryData);

private:
	unsigned int Length,Width,Displaced;
	int Elevation;
	double MagBearing;		/* Magnetic bearing */
	double Lon,Lat;
	double Gradient;
	std::string Notes;
	std::string ID;
	std::string AirportICAO;
	std::vector<NavAid*> NavAids;
};

/* Increment the version when the structure changes */
#define RUNWAY_BINARY_STRUCT_VER 1
struct Runway_Binary_struct {
	uint16_t Length,Width,Displaced;
	int16_t Elevation;
	uint16_t MagBearingx10;
	int32_t Lonx1000000,Latx1000000;
	int16_t Gradientx10;
	char Notes[30];
	char ID[10];
} __attribute__ ((packed));

#endif /*RUNWAY_H_*/
